@@ -86,6 +86,16 @@ typedef struct {
     /* */
   int        useDotNames;
 
+
+    /* The XML context */
+  xmlParserCtxtPtr ctx;
+
+    /* A function which is used to process a branch "anonymously, i.e
+       not one that is actively identified in the branches = list(....)
+       but a function that is returned from a regular startElement handler
+       that indicates collect up the node and call this.
+     */
+  USER_OBJECT_ dynamicBranchFunction;
 } RS_XMLParserData;
 
 /* The name of the R element to call fo the general case.
@@ -127,8 +137,8 @@ struct NodeList {
 /* Allocate a data structure for use with the parser */
 RS_XMLParserData *createRSXMLParserData(USER_OBJECT_ handlers) ;
 
-USER_OBJECT_ RS_XML(callUserFunction)(char *opName, const char *preferredName, RS_XMLParserData *parser, USER_OBJECT_ opArgs) ;
-USER_OBJECT_ RS_XML(createAttributesList)(const char **atts);
+USER_OBJECT_ RS_XML(callUserFunction)(const char *opName, const char *preferredName, RS_XMLParserData *parser, USER_OBJECT_ opArgs) ;
+/*Made static now: USER_OBJECT_ RS_XML(createAttributesList)(const char **atts); */
 
 
 void RS_XML(entityDeclarationHandler)(void *userData, const XML_Char *entityName, 
@@ -154,7 +164,7 @@ RS_XMLParserData *RS_XML(createParserData)(USER_OBJECT_ handlers);
 int RS_XML(parseBufferWithParserData)(char *buf, RS_XMLParserData *parserData);
 int RS_XML(notStandAloneHandler)(void *userData);
 
-void RS_XML(libXMLEventParse)(const char *fileName, RS_XMLParserData *parserData, RS_XML_ContentSourceType asText,
+int RS_XML(libXMLEventParse)(const char *fileName, RS_XMLParserData *parserData, RS_XML_ContentSourceType asText,
 			      int saxVersion);
 
 
